# Generated by Django 4.2.18 on 2025-01-16 15:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0031_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='smstemplate',
            name='sender_name',
            field=models.CharField(blank=True, max_length=20, null=True),
        ),
    ]
