# Generated by Django 4.2.25 on 2025-10-12 20:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0033_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='outputsmsmessage',
            name='is_voice_message',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='smstemplate',
            name='is_voice_message',
            field=models.BooleanField(default=False),
        ),
    ]
