# Generated by Django 4.2.23 on 2025-08-06 10:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0032_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailtemplate',
            name='version',
            field=models.CharField(verbose_name='version', blank=True, max_length=10, null=True),
        ),
    ]
