# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-07-26 12:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0024_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='dialertemplate',
            name='is_locked',
            field=models.BooleanField(default=False, verbose_name='is locked'),
        ),
        migrations.AddField(
            model_name='emailtemplate',
            name='is_locked',
            field=models.BooleanField(default=False, verbose_name='is locked'),
        ),
        migrations.AddField(
            model_name='pushnotificationtemplate',
            name='is_locked',
            field=models.BooleanField(default=False, verbose_name='is locked'),
        ),
        migrations.AddField(
            model_name='smstemplate',
            name='is_locked',
            field=models.BooleanField(default=False, verbose_name='is locked'),
        ),
    ]
