# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-12-10 16:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0010_20191120_1002'),
    ]

    operations = [
        migrations.AddField(
            model_name='dialermessage',
            name='is_autodialer',
            field=models.BooleanField(default=True, verbose_name='is autodialer'),
        ),
        migrations.AlterField(
            model_name='dialermessage',
            name='content',
            field=models.TextField(blank=True, null=True, verbose_name='content'),
        ),
        migrations.AlterField(
            model_name='dialermessage',
            name='state',
            field=models.IntegerField(choices=[(0, 'not assigned'), (1, 'ready'), (2, 'rescheduled by dialer'), (3, 'call in progress'), (4, 'hangup'), (5, 'done'), (6, 'rescheduled'), (7, 'listened up complete message'), (8, 'listened up partial message'), (9, 'unreachable'), (10, 'declined'), (11, 'unanswered'), (12, 'answered - hangup by dialer'), (13, 'answered - hangup by customer'), (66, 'error'), (77, 'debug')], editable=False, verbose_name='state'),
        ),
    ]
