# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2019-11-08 20:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0008_auto_20190726_1459'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailmessage',
            name='require_pull_info',
            field=models.BooleanField(default=False, verbose_name='require to pull message info from email service'),
        ),
    ]
